# This file is part of Build Your Own CI
#
# Copyright 2017, 2018 Vincent Ladeuil
# Copyright 2016, 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re

from byot import features

HERE = os.path.abspath(os.path.dirname(__file__))

lxd_client_feature = features.ExecutableFeature('lxc')
make_feature = features.ExecutableFeature('make')
flake8_feature = features.ExecutableFeature('flake8')


class OnHost(features.Feature):
    """Is the current process running on a host whose name matches a regexp."""

    def __init__(self, hostname_regexp):
        super().__init__()
        self.hostname_regexp = hostname_regexp
        self.compiled_re = re.compile(hostname_regexp)

    def _probe(self):
        return self.compiled_re.search(os.uname().nodename) is not None

    def feature_name(self):
        return 'Running on a hostname matching {}'.format(self.hostname_regexp)

on_monitor = OnHost('^brz-monitor')
on_slave = OnHost('^brz-slave-')


# Features relying on an environment property should be checked before any test
# isolation is setup. Once this is done, they can be freely used to decorate
# tests [classes] or during setUp().

lxd_client_feature.available()
make_feature.available()
flake8_feature.available()
on_monitor.available()
on_slave.available()

# Useful shortcuts to export but not use internally
ExecutableFeature = features.ExecutableFeature
Feature = features.Feature
requires = features.requires
test_requires = features.test_requires
