# This file is part of Build Your Own CI
#
# Copyright 2018 Vincent Ladeuil
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import io
import unittest

from byoci.host import (
    commands,
    config,
)
from byoci.tests.host import fixtures


class TestMonitorConfig(unittest.TestCase):
    """Test master config."""

    def setUp(self):
        super().setUp()
        fixtures.isolate_from_disk_for_host(self)

    def test_defaults(self):
        conf = config.VmStack(None)
        self.assertEqual('selftest', conf.get('byoci'))
        self.assertEqual(self.monitor_name,
                         conf.expand_options('{{byoci}.monitor}'))
        self.assertEqual('/byoci',
                         conf.expand_options('{{byoci}.definition}'))


class TestMonitorSetup(unittest.TestCase):

    def setUp(self):
        super().setUp()
        fixtures.isolate_from_disk_for_host(self)
        fixtures.setup_byoci_conf(self)
        self.out = io.StringIO()
        self.err = io.StringIO()
        fixtures.override_logging(self)

    def test_setup_succeeds(self):
        fixtures.setup_master(self)
        fixtures.setup_monitor(self)
        master_conf = config.VmStack(self.master_name)
        monitor_conf = config.VmStack(self.monitor_name)
        # Ensure monitor knows where to talk to master
        self.assertEqual(master_conf.get('{byoci}.master.api'),
                         monitor_conf.get('{byoci}.master.api'))


class TestMonitorUpdateJobs(unittest.TestCase):

    def setUp(self):
        super().setUp()
        fixtures.isolate_from_disk_for_host(self)
        fixtures.setup_byoci_conf(self)
        self.out = io.StringIO()
        self.err = io.StringIO()
        fixtures.override_logging(self)

    def test_succeeds(self):
        fixtures.setup_master(self)
        fixtures.setup_monitor(self)
        shell_command = commands.Shell(out=self.out, err=self.err)
        # We can't capture :-/ So redirect to a file
        # FIXME: Assert something about the file content after downloading it
        # -- vila 2018-02-09
        shell_command.parse_args([self.monitor_name,
                                  'cd /byoci && jenkins-jobs update jobs/'
                                  ' > ~/jobs.log 2>&1'])
        ret = shell_command.run()
        self.assertEqual(0, ret)


class TestMonitorUpdatePlugins(unittest.TestCase):

    def setUp(self):
        super().setUp()
        fixtures.isolate_from_disk_for_host(self)
        fixtures.setup_byoci_conf(self)
        self.out = io.StringIO()
        self.err = io.StringIO()
        fixtures.override_logging(self)

    def test_succeeds(self):
        fixtures.setup_master(self)
        fixtures.setup_monitor(self)
        shell_command = commands.Shell(out=self.out, err=self.err)
        # We can't capture :-/ So redirect to a file

        # FIXME: Assert something about the file content after downloading it.
        # And/or assert something about the plugins being updated on master.
        # -- vila 2018-02-09
        shell_command.parse_args(
            [self.monitor_name,
             'cd /byoci && ./byo-ci-monitor update-plugins'])
        ret = shell_command.run()
        self.assertEqual(0, ret)
