#!/usr/bin/env python3

# FIXME: Ideally byov will learn to import and execute directly
# -- vila 2018-02-17

# This file is part of Build Your Own CI
#
# Copyright 2018 Vincent Ladeuil
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

# /!\ This is expanded by byov, caveat emptor

import logging
import os
import sys
import tempfile


logger = logging.getLogger(__name__)
sys.path.insert(0, '{{byoci}.host.definition}')


# FIXME: Come on... I just set sys.path above so the following import can
# succeed, surely pep8 shouldn't complain ? -- vila 2018-02-17
from byoci.host import (  # noqa
    config,
    commands,
)


def master_knows_slave():
    logger.debug('Telling master about {vm.name}')
    sconf = config.VmStack('{vm.name}')
    kls = sconf.get('vm.class')
    slave = kls(sconf)
    # Getting the key from the slave itself should avoid MITM attacks ;-)
    ret, out, err = slave.shell_captured('ssh-keyscan -t rsa localhost')
    host_def = '"' + out.replace('localhost ', '{vm.name},{vm.ip} ') + '"'
    logger.debug('Will add {} to master known_hosts'.format(host_def))
    master_name = '{{byoci}.master}'
    commands.run_from_hook(
        ['shell', master_name, 'ssh-keygen -R {vm.name} || true'])
    commands.run_from_hook(
        ['shell', master_name,
         'echo', '-n', host_def, '>>', '~/.ssh/known_hosts'])
    # Add the proxy in master known_hosts if needed.
    auth_proxy = sconf.get('ssh.proxy.known_host')
    if auth_proxy:
        commands.run_from_hook(
            ['shell', master_name,
             'echo', auth_proxy, '>>', '~/.ssh/known_hosts'])
    # Create slave profile for ssh access
    tmp = tempfile.NamedTemporaryFile(delete=False, mode='w')
    tmp.write('#!/bin/sh -x\n')
    proxy_cmd = sconf.get('ssh.proxy.command')
    if proxy_cmd:
        tmp.write('SLAVE_SSH_OPTS="-oProxyCommand={}"\n'.format(
            proxy_cmd))
    tmp.close()
    logger.debug('tmp.name: {}'.format(tmp.name))
    try:
        # Strictly speaking we should ask the master but jenkins.home is shared
        dst = os.path.join(sconf.get('jenkins.home'), 'bin', '{vm.name}')
        commands.run_from_hook(['push', master_name, tmp.name, dst])
    finally:
        os.remove(tmp.name)


def upload_localhost_config():
    # Upload a config file for the slave so it can expand scripts
    logger.debug('Uploading the localhost config')
    option_set = commands.matching_options(
        '{vm.name}', ['^byoci$', '^{byoci}\.', '^launchpad\.'])
    tmp = tempfile.NamedTemporaryFile(delete=False, mode='w')
    tmp.write('[localhost]\n')
    for name in sorted(option_set.keys()):
        tmp.write('{} = {}\n'.format(name, option_set[name]))
    tmp.close()
    logger.debug('tmp.name: {}'.format(tmp.name))
    localhost_dir = os.path.join('~', '.config', 'byov', 'conf.d')
    localhost_path = os.path.join(localhost_dir, 'byov-localhost.conf')
    try:
        commands.run_from_hook(
            ['shell', '{vm.name}', 'mkdir', '-p', localhost_dir])
        commands.run_from_hook(['push', '{vm.name}', tmp.name, localhost_path])
    finally:
        os.remove(tmp.name)


if __name__ == '__main__':
    commands.setup_logging('{logging.level}'.upper())
    secrets = commands.SetupSecrets('{byoci}')
    # Add master key to slave authorized_keys
    logger.debug('Allowing master on {vm.name}')
    name, path = secrets.master_key()
    pub = commands.push_ssh_file('{vm.name}', name + '.pub', path + '.pub')
    auth_keys = os.path.join('~', '.ssh', 'authorized_keys')
    commands.run_from_hook(
        ['shell', '{vm.name}', 'cat {} >> {}'.format(pub, auth_keys)])
    # Add slave server key to master known_hosts
    master_knows_slave()
    # Install slave key for launchpad write access
    name, path = secrets.slave_key()
    commands.push_ssh_file('{vm.name}', name, path)
    commands.push_ssh_file('{vm.name}', name + '.pub', path + '.pub')
    upload_localhost_config()
